/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import javax.swing.*;
import javax.swing.event.*;
import EDU.auburn.VGJ.util.DPoint;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;

/**
 * A panel containing an AngleControl, along with buttons for switching to the
 * XY plane, YZ plane and XZ plane.
 *
 * @author Dennis Wagelaar
 * @version 0.0.12, 17 March 2002
 */

public class AngleControlPanel extends JPanel
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private AngleControl angleControl = new AngleControl();
  /**
   * Only one <code>ChangeEvent</code> is needed per instance since the
   * event's only state is the source property.  The source of events
   * generated is always "this".
   */
  protected transient ChangeEvent changeEvent;
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel angleControlPanel = new JPanel();
  JPanel angleButtonPanel = new JPanel();
  JLabel jLabel1 = new JLabel();
  JButton xyBtn = new JButton();
  JButton xzBtn = new JButton();
  JButton yzBtn = new JButton();
  FlowLayout flowLayout2 = new FlowLayout();
  BorderLayout borderLayout2 = new BorderLayout();
  Border border1;

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs an AngleControlPanel
   */
  public AngleControlPanel()
  {
    super();
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  } //AngleControlPanel(int width, int height)

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  private void jbInit() throws Exception
  {
    border1 = BorderFactory.createEmptyBorder(5,5,5,5);
    this.setLayout(borderLayout1);
    jLabel1.setText("Plane:");
    xyBtn.setMargin(new Insets(2, 8, 2, 8));
    xyBtn.setText("XY");
    xyBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        xyBtn_actionPerformed(e);
      }
    });
    xzBtn.setMargin(new Insets(2, 8, 2, 8));
    xzBtn.setText("XZ");
    xzBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        xzBtn_actionPerformed(e);
      }
    });
    yzBtn.setMargin(new Insets(2, 8, 2, 8));
    yzBtn.setText("YZ");
    yzBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        yzBtn_actionPerformed(e);
      }
    });
    angleControlPanel.setLayout(borderLayout2);
    angleButtonPanel.setLayout(flowLayout2);
    angleControl.addChangeListener(new javax.swing.event.ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        angleControl_stateChanged(e);
      }
    });
    angleControlPanel.setBorder(border1);
    this.add(angleControlPanel,  BorderLayout.CENTER);
    this.add(angleButtonPanel,  BorderLayout.SOUTH);
    angleButtonPanel.add(jLabel1, null);
    angleButtonPanel.add(xyBtn, null);
    angleButtonPanel.add(xzBtn, null);
    angleButtonPanel.add(yzBtn, null);
    angleControlPanel.add(angleControl, BorderLayout.CENTER);
  } //jbInit

  /* XY button pressed. */
  void xyBtn_actionPerformed(ActionEvent e)
  {
    setAngles(new DPoint(0.0, Math.PI / 2.0));
    // post event
    fireStateChanged();
  } //xyBtn_actionPerformed

  /* XZ button pressed. */
  void xzBtn_actionPerformed(ActionEvent e)
  {
    setAngles(new DPoint(0.0, 0.0));
    // post event
    fireStateChanged();
  } //xzBtn_actionPerformed

  /* YZ button pressed. */
  void yzBtn_actionPerformed(ActionEvent e)
  {
    setAngles(new DPoint(Math.PI / 2.0, 0.0));
    // post event
    fireStateChanged();
  } //yzBtn_actionPerformed

  /* The state of the AngleControl has changed. */
  void angleControl_stateChanged(ChangeEvent e)
  {
    // forward event
    fireStateChanged();
  } //angleControlstateChanged

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Notifies all listeners that have registered interest for
   * notification on this event type.  The event instance
   * is lazily created using the parameters passed into
   * the fire method.
   *
   * @see EventListenerList
   */
  protected void fireStateChanged()
  {
    // Guaranteed to return a non-null array
    Object[] listeners = listenerList.getListenerList();
    // Process the listeners last to first, notifying
    // those that are interested in this event
    for (int i = listeners.length - 2; i >= 0; i -= 2)
    {
      if (listeners[i] == ChangeListener.class)
      {
        // Lazily create the event:
        if (changeEvent == null)
          changeEvent = new ChangeEvent(this);
        ((ChangeListener) listeners[i + 1]).stateChanged(changeEvent);
      }
    }
  } //fireStateChanged

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Gets the angles attribute.
   *
   * @return The angles of the AngleControl
   */
  public DPoint getAngles()
  {
    return angleControl.getAngles();
  } //getAngles

  /**
   * Sets the angles attribute.
   *
   * @param angles The angles of the AngleControl
   */
  public void setAngles(DPoint angles)
  {
    angleControl.setAngles(angles.x, angles.y);
  } //setAngles

  /**
   * Makes the component visible or invisible. Overrides Component.setVisible.
   *
   * @param aFlag True if and only if the component should be visible.
   */
  public void setVisible(boolean aFlag)
  {
    super.setVisible(aFlag);
    angleControl.setVisible(aFlag);
  } //setVisible

  /**
   * Returns whether the mouse is dragging in the AngleControl.
   *
   * @return True if and only if the mouse is dragging in the AngleControl.
   */
  public boolean isDragging()
  {
    return angleControl.isDragging();
  } //isDragging

  /**
   * Adds a ChangeListener to the viewport scroller.
   *
   * @param l the listener to be added
   */
  public void addChangeListener(ChangeListener l)
  {
    listenerList.add(ChangeListener.class, l);
  } //addChangeListener

  /**
   * Removes a ChangeListener from the viewport scroller.
   *
   * @param l the listener to be removed
   */
  public void removeChangeListener(ChangeListener l)
  {
    listenerList.remove(ChangeListener.class, l);
  } //removeChangeListener
} //class AngleControlPanel
