/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.awt.*;
import javax.swing.JDialog;
import EDU.utwente.CoCompose.lang.AbstractConcept;

/**
 * Abstract CoCompose dialog class. Introduces error reporting and window
 * centering functionality.
 *
 * @author Dennis Wagelaar
 * @version 0.0.28, 27 June 2002
 */

public class CoComposeDialog extends JDialog
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private Frame frame_;

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a CoComposeDialog.
   *
   * @param frame The parent frame.
   * @param title The dialog title.
   * @param modal Shows the dialog in modal mode when true.
   */
  public CoComposeDialog(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);
    frame_ = frame;
  } //CoComposeDialog(Frame frame, String title, boolean modal)

  /**
   * Constructs a CoComposeDialog.
   * This constructor is only for the JBuilder Designer, do not use it!
   */
  public CoComposeDialog()
  {
    this(null, "", false);
  } //CoComposeDialog()

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Center window position relative to parent frame, if any.
   */
  protected void centerPosition()
  {
    if (frame_ != null)
    {
      Dimension frameSize = frame_.getSize();
      Point framePos = frame_.getLocationOnScreen();
      Dimension size = getPreferredSize();
      if (size.height > frameSize.height)
        size.height = frameSize.height;
      if (size.width > frameSize.width)
        size.width = frameSize.width;
      setLocation(framePos.x + (frameSize.width - size.width) / 2,
        framePos.y + (frameSize.height - size.height) / 2);
    }
  } //centerPosition

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Reports an error.
   *
   * @param message The error message.
   * @param type The message type (Error, Warning or Info).
   * @param concept The error subject concept, if any.
   */
  public void reportError(String message, String type, AbstractConcept concept)
  {
    if (concept != null)
      concept.setFillColor(Color.red);
    MessageDialog errDlg = new MessageDialog(frame_, type, true);
    // switch type
    if (type.equals("Error"))
      errDlg.setIcon(MessageDialog.ICON_ERROR);
    else if (type.equals("Warning"))
      errDlg.setIcon(MessageDialog.ICON_WARNING);
    else if (type.equals("Info"))
      errDlg.setIcon(MessageDialog.ICON_INFO);
    errDlg.setMessage(message);
    errDlg.show();
  } //reportError

  /**
   * Accepts generic notification message.
   *
   * @param sender The message sender.
   */
  public void notify(Object sender)
  {
  } //notify

  /**
   * Shows dialog window.
   */
  public void show()
  {
    centerPosition();
    super.show();
  } //show
} //class CoComposeDialog
