/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * A standard message dialog for displaying error, warning and info messages.
 *
 * @author Dennis Wagelaar
 * @version 0.1.03, 24 October 2002
 */

public class MessageDialog extends JDialog
{
  //////////////////////////////////////////////////////////////////////////////
  // Constants
  //////////////////////////////////////////////////////////////////////////////

  /**
   * The icon value for an error icon.
   */
  public static final int ICON_ERROR   = 0;
  /**
   * The icon value for a warning icon.
   */
  public static final int ICON_WARNING = 1;
  /**
   * The icon value for an info icon.
   */
  public static final int ICON_INFO    = 2;
  /**
   * The button value for an ok button.
   */
  public static final int BUTTON_OK     = 0;
  /**
   * The button value for a cancel button.
   */
  public static final int BUTTON_CANCEL = 1;
  /**
   * The button value for a yes button.
   */
  public static final int BUTTON_YES    = 2;
  /**
   * The button value for a no button.
   */
  public static final int BUTTON_NO     = 3;
  /**
   * The button value for an abort button.
   */
  public static final int BUTTON_ABORT  = 4;
  /**
   * The button value for a retry button.
   */
  public static final int BUTTON_RETRY  = 5;
  /**
   * The button value for an ignore button.
   */
  public static final int BUTTON_IGNORE = 6;

  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private Frame frame_;
  private int buttonPressed_;
  ImageIcon errorIcon = new ImageIcon("./images/error.gif");
  ImageIcon warningIcon = new ImageIcon("./images/warning.gif");
  ImageIcon infoIcon = new ImageIcon("./images/info.gif");
  JPanel panel1 = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanel2 = new JPanel();
  JPanel jPanel3 = new JPanel();
  JButton okBtn = new JButton();
  FlowLayout flowLayout2 = new FlowLayout();
  JButton cancelBtn = new JButton();
  JButton yesBtn = new JButton();
  JButton noBtn = new JButton();
  JButton abortBtn = new JButton();
  JButton retryBtn = new JButton();
  JButton ignoreBtn = new JButton();
  JPanel jPanel1 = new JPanel();
  JPanel jPanel4 = new JPanel();
  JLabel messageLabel = new JLabel();
  JLabel messageLabel2 = new JLabel();
  BorderLayout borderLayout2 = new BorderLayout();
  FlowLayout msgLabelLayout = new FlowLayout();
  FlowLayout msgLabel2layout = new FlowLayout();
  JPanel jPanel5 = new JPanel();
  JLabel iconLabel = new JLabel();
  FlowLayout flowLayout4 = new FlowLayout();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a MessageDialog.
   *
   * @param frame The parent frame.
   * @param title The dialog title.
   * @param modal Shows the dialog in modal mode when true.
   */
  public MessageDialog(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);
    frame_ = frame;
    try
    {
      jbInit();
      mInit();
      pack();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  } //MessageDialog(Frame frame, String title, boolean modal)

  /**
   * Constructs a MessageDialog.
   * This constructor is only for the JBuilder Designer, do not use it!
   */
  MessageDialog()
  {
    this(null, "", false);
  } //MessageDialog()

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  void jbInit() throws Exception
  {
    panel1.setLayout(borderLayout1);
    okBtn.setText("OK");
    okBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        okBtn_actionPerformed(e);
      }
    });
    jPanel2.setLayout(borderLayout2);
    jPanel3.setLayout(flowLayout2);
    panel1.setBorder(BorderFactory.createEtchedBorder());
    cancelBtn.setText("Cancel");
    cancelBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        cancelBtn_actionPerformed(e);
      }
    });
    yesBtn.setText("Yes");
    yesBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        yesBtn_actionPerformed(e);
      }
    });
    noBtn.setText("No");
    noBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        noBtn_actionPerformed(e);
      }
    });
    abortBtn.setText("Abort");
    abortBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        abortBtn_actionPerformed(e);
      }
    });
    retryBtn.setText("Retry");
    retryBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        retryBtn_actionPerformed(e);
      }
    });
    ignoreBtn.setText("Ignore");
    ignoreBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        ignoreBtn_actionPerformed(e);
      }
    });
    jPanel2.setMaximumSize(new Dimension(500, 500));
    jPanel1.setLayout(msgLabelLayout);
    jPanel4.setLayout(msgLabel2layout);
    jPanel5.setLayout(flowLayout4);
    msgLabel2layout.setVgap(0);
    msgLabelLayout.setVgap(0);
    getContentPane().add(panel1);
    panel1.add(jPanel2,  BorderLayout.CENTER);
    jPanel2.add(jPanel1, BorderLayout.NORTH);
    jPanel1.add(messageLabel, null);
    jPanel2.add(jPanel4, BorderLayout.CENTER);
    jPanel4.add(messageLabel2, null);
    panel1.add(jPanel3, BorderLayout.SOUTH);
    jPanel3.add(okBtn, null);
    jPanel3.add(cancelBtn, null);
    jPanel3.add(yesBtn, null);
    jPanel3.add(noBtn, null);
    jPanel3.add(abortBtn, null);
    jPanel3.add(retryBtn, null);
    jPanel3.add(ignoreBtn, null);
    panel1.add(jPanel5, BorderLayout.WEST);
    jPanel5.add(iconLabel, null);
  } //jbInit

  /* Manual initialization. */
  private void mInit()
  {
    // hide non-standard buttons
    cancelBtn.setVisible(false);
    yesBtn.setVisible(false);
    noBtn.setVisible(false);
    abortBtn.setVisible(false);
    retryBtn.setVisible(false);
    ignoreBtn.setVisible(false);
  } //mInit

  /* Center window position relative to parent frame, if any. */
  private void centerPosition()
  {
    if (frame_ != null)
    {
      Dimension frameSize = frame_.getSize();
      Point framePos = frame_.getLocationOnScreen();
      Dimension size = getPreferredSize();
      if (size.height > frameSize.height)
        size.height = frameSize.height;
      if (size.width > frameSize.width)
        size.width = frameSize.width;
      setLocation(framePos.x + (frameSize.width - size.width) / 2,
        framePos.y + (frameSize.height - size.height) / 2);
    }
  } //centerPosition

  /* OK button pressed. */
  void okBtn_actionPerformed(ActionEvent e)
  {
    buttonPressed_ = BUTTON_OK;
    dispose();
  } //okBtn_actionPerformed

  /* Cancel button pressed. */
  void cancelBtn_actionPerformed(ActionEvent e)
  {
    buttonPressed_ = BUTTON_CANCEL;
    dispose();
  } //cancelBtn_actionPerformed

  /* Yes button pressed. */
  void yesBtn_actionPerformed(ActionEvent e)
  {
    buttonPressed_ = BUTTON_YES;
    dispose();
  } //yesBtn_actionPerformed

  /* No button pressed. */
  void noBtn_actionPerformed(ActionEvent e)
  {
    buttonPressed_ = BUTTON_NO;
    dispose();
  } //noBtn_actionPerformed

  /* Abort button pressed. */
  void abortBtn_actionPerformed(ActionEvent e)
  {
    buttonPressed_ = BUTTON_ABORT;
    dispose();
  } //abortBtn_actionPerformed

  /* Retry button pressed. */
  void retryBtn_actionPerformed(ActionEvent e)
  {
    buttonPressed_ = BUTTON_RETRY;
    dispose();
  } //retryBtn_actionPerformed

  /* Ignore button pressed. */
  void ignoreBtn_actionPerformed(ActionEvent e)
  {
    buttonPressed_ = BUTTON_IGNORE;
    dispose();
  } //ignoreBtn_actionPerformed

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Sets the icon in the dialog, given one of the icon values.
   *
   * @param icon The icon value (ICON_ERROR | ICON_WARNING | ICON_INFO).
   */
  public void setIcon(int icon)
  {
    switch (icon)
    {
      case ICON_ERROR:
        iconLabel.setIcon(errorIcon);
        break;
      case ICON_WARNING:
        iconLabel.setIcon(warningIcon);
        break;
      case ICON_INFO:
        iconLabel.setIcon(infoIcon);
        break;
    }
    pack();
  } //setIcon

  /**
   * Sets the dialog message.
   *
   * @param message The dialog message.
   */
  public void setMessage(String message)
  {
    int splitIndex;

    if (message.length() > 100)
    { // split message over two lines
      splitIndex = message.indexOf(' ', Math.round(message.length() / 2));
      if (splitIndex == -1)
        splitIndex = Math.round(message.length() / 2);
      messageLabel.setText(message.substring(0, splitIndex).trim());
      messageLabel2.setText(message.substring(splitIndex).trim());
      msgLabelLayout.setVgap(0);
    }
    else
    { // one line
      messageLabel.setText(message);
      messageLabel2.setText("");
      msgLabelLayout.setVgap(11);
    }
    pack();
  } //setMessage

  /**
   * Sets the buttons that should appear in the dialog.
   *
   * @param buttons The array of button IDs.
   */
  public void setButtons(int[] buttons)
  {
    // hide standard buttons
    okBtn.setVisible(false);
    // make all selected buttons visible
    for (int index = 0; index < buttons.length; index++)
    {
      switch (buttons[index])
      {
        case BUTTON_OK:
          okBtn.setVisible(true);
          break;
        case BUTTON_CANCEL:
          cancelBtn.setVisible(true);
          break;
        case BUTTON_YES:
          yesBtn.setVisible(true);
          break;
        case BUTTON_NO:
          noBtn.setVisible(true);
          break;
        case BUTTON_ABORT:
          abortBtn.setVisible(true);
          break;
        case BUTTON_RETRY:
          retryBtn.setVisible(true);
          break;
        case BUTTON_IGNORE:
          ignoreBtn.setVisible(true);
          break;
      }
    }
  } //setButtons

  /**
   * Gets the id of the button that was pressed.
   *
   * @return The id of the button that was pressed.
   */
  public int getPressedButton()
  {
    return buttonPressed_;
  } //getPressedButton

  /**
   * Shows the window.
   */
  public void show()
  {
    centerPosition();
    Toolkit.getDefaultToolkit().beep();
    super.show();
  } //show
} //class MessageDialog
