/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2004
 * Author:       Willem Alexander Hajenius
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: Library.java,v 1.2 2004/05/04 13:45:12 dwagelaa Exp $
 * $Log: Library.java,v $
 * Revision 1.2  2004/05/04 13:45:12  dwagelaa
 * updated library submitted at meeting with Willem on 4-may-2004
 *
 * Revision 1.1  2004/04/27 12:52:24  dwagelaa
 * added preliminary CoCompose Library support.
 *
 */

package EDU.utwente.CoCompose.util;

import EDU.utwente.CoCompose.lang.*;
import EDU.utwente.CoCompose.translate.*;
import java.util.*;
import java.util.regex.*;
import java.io.*;
import java.sql.*;
import java.util.regex.*;
import javax.swing.ProgressMonitor;



/**
 * CoCompose component library based on a relational database.
 * @author Willem Hajenius
 * @version 0.1.22, 27-apr-2004
 * @since 0.1.22
 */
public class Library
{
  class CRQL
  {
    public String propname, propvalue, tablename;
    public int type, limit = Library.INF_DEPTH;
    public double weight = 1;
    public CRQL () {}
    public CRQL (String pn, int t, String pv, double w, int l)
    {
      propname = pn; type = t; propvalue = pv; weight = w; limit = l;
    }
    public String toString()
    {
      return "(Table " + tablename + "): " + propname +
        (type == Library.ATLEAST ? " ATLEAST " : " ATMOST ") + propvalue +
        " WEIGHT " + weight + (limit == Library.INF_DEPTH ? " (no limit)" : " LIMIT " + limit);
    }
  }


  class RankComparator implements Comparator
  {
    private AbstractMap ranks;
    private int uriIndex;

    public RankComparator (AbstractMap r, int i)
    { ranks = r; uriIndex = i;}

    public int compare (Object o1, Object o2)
    {
      String uri1 = (String) ((Vector) o1).get (uriIndex);
      String uri2 = (String) ((Vector) o2).get (uriIndex);
      double rank1 = ((Double) ranks.get (uri1)).doubleValue();
      double rank2 = ((Double) ranks.get (uri2)).doubleValue();
      if (rank1 > rank2) return -1; 
      else if (rank1 < rank2) return 1;
      else return 0;
    }
  }

            
  class Indexer extends Thread
  {
    private static final String UPDATE_URI = "LIBRARY";
    private static final String UPDATE_PROPNAME = "LastIndexedAt";
    List files;
    boolean updateAll;

    public Indexer (List files)
    {
      this.files = files;
      this.updateAll = true;
    }

    public Indexer (List files, boolean updateAll)
    {
      this.files = files;
      this.updateAll = updateAll;
    }
    
    public void run()
    {
      // Index each of these components
      ListIterator li = files.listIterator();
      ProgressMonitor pm = new ProgressMonitor(null, "Indexing component:", "?", 0, files.size());
      pm.setMillisToDecideToPopup(100);
      pm.setMillisToPopup(200);
      Vector v = sqlQuerySimple ("SELECT propvalue FROM Properties WHERE uri = '" + UPDATE_URI
        + "' AND propname = '" + UPDATE_PROPNAME + "';", "propvalue");
      long lastupdate = 0;
      if (v.size() > 0) lastupdate = Long.valueOf((String) v.get(0)).longValue();

      sqlUpdateBlind ("DELETE FROM Properties WHERE uri = '" + UPDATE_URI
        + "' AND propname = '" + UPDATE_PROPNAME + "';");
      sqlUpdateBlind ("INSERT INTO Properties VALUES ('" + UPDATE_URI + "', '" + UPDATE_PROPNAME
        + "', '" + Long.toString(System.currentTimeMillis()) + "');");

      int n = 0;
      while (li.hasNext())
      {
        File f = (File) li.next();
        if (updateAll || f.lastModified() > lastupdate)
        {
          pm.setNote (f.getPath());
          Library.instance().indexComponent (f);
        }
        pm.setProgress (++n);
      }
      pm.close();
    }
  }


  /** Constants */
  public static final String databaseName = "library/ComponentLibrary";
  public static final String tableLibraryFilename = "library.csv";
  public static final String tablePropertiesFilename = "props.csv";
  public static final String tableTaxonomyFilename = "taxonomy.csv";
  public static final String tableSavedQueriesFilename = "queries.csv";
  public static final int INF_DEPTH = -1;
  public static final int ATLEAST = 200;
  public static final int ATMOST = 300;

  protected static Connection conn = null;
  protected static PrintStream log  = System.out;
  private static Library instance = null;


  /**
   * Stand-alone debugging method
   * @param args Command-line arguments
   */
  public static void main (String[] args)
  {
    Library lib = Library.instance();
    try
    {
      System.out.println ("Testing CoCompose library:");

      lib.sqlUpdateBlind ("DELETE FROM Taxonomy;");
      lib.sqlUpdateBlind ("DELETE FROM Library;");
      lib.sqlUpdateBlind ("DELETE FROM Properties;");

      lib.sqlUpdateBlind ("INSERT INTO Taxonomy VALUES ('quality.adaptability', 'fixed', NULL);");
      lib.sqlUpdateBlind ("INSERT INTO Taxonomy VALUES ('quality.adaptability', 'compile-time', 'fixed');");
      lib.sqlUpdateBlind ("INSERT INTO Taxonomy VALUES ('quality.adaptability', 'run-time', 'compile-time');");
      lib.sqlUpdateBlind ("INSERT INTO Taxonomy VALUES ('platform.vm', 'java', NULL);");
      lib.sqlUpdateBlind ("INSERT INTO Taxonomy VALUES ('platform.vm', 'personal-java', 'java');");
      lib.sqlUpdateBlind ("INSERT INTO Taxonomy VALUES ('platform.vm', 'java2', 'java');");
      lib.sqlUpdateBlind ("INSERT INTO Taxonomy VALUES ('platform.vm', 'j2me', 'java2');");
      lib.sqlUpdateBlind ("INSERT INTO Taxonomy VALUES ('software.java.graphics', 'awt', NULL);");
      lib.sqlUpdateBlind ("INSERT INTO Taxonomy VALUES ('software.java.graphics', 'swing', 'awt');");

      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-A', 'quality.adaptability', 'fixed');");
      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-A', 'platform.vm', 'personal-java');");
      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-A', 'software.java.graphics', 'swing');");
      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-B', 'quality.adaptability', 'run-time');");
      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-B', 'platform.vm', 'java');");
      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-B', 'software.java.graphics', 'swing');");
      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-C', 'quality.adaptability', 'compile-time');");
      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-C', 'platform.vm', 'java');");
      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-C', 'software.java.graphics', 'swing');");
      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-D', 'quality.adaptability', 'compile-time');");
      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-D', 'platform.vm', 'personal-java');");
      lib.sqlUpdateBlind ("INSERT INTO Properties VALUES ('uri-D', 'software.java.graphics', 'awt');");

      lib.sqlUpdateBlind ("INSERT INTO Library VALUES ('uri-A', 'URI-A', 'concept');");
      lib.sqlUpdateBlind ("INSERT INTO Library VALUES ('uri-B', 'URI-B', 'composite-type');");
      lib.sqlUpdateBlind ("INSERT INTO Library VALUES ('uri-C', 'URI-C', 'solution-pattern');");
      lib.sqlUpdateBlind ("INSERT INTO Library VALUES ('uri-D', 'URI-D', 'solution-pattern');");

      String query = "SELECT DISTINCT P1.uri FROM Properties P1,    Properties P2, Properties P3 " +
                    " WHERE P1.uri = P2.uri AND P2.uri = P3.uri" +
                    " AND (P1.propname = 'quality.adaptability' AND P1.propvalue ATLEAST 'fixed' WEIGHT 2)" +
                    " AND ((P2.propname = 'platform.vm' AND P2.propvalue ATMOST 'personal-java' LIMIT 5)" +
                    " OR (P3.propname = 'software.java.graphics' AND P3.propvalue ATLEAST 'awt' WEIGHT 2 LIMIT 9));";

      Vector test = lib.query (query);
      if (test == null) { log.println("Result table is null!"); }
      else for (int i = 0; i < test.size(); i++)
      {
        Vector row = (Vector) test.elementAt(i);
        System.out.println ("Row " + i + ": " + row);
      }

      lib.shutdown();
      System.out.println ("Done.\n");
    }
    catch (Exception e)
    {
      log.println ("Caught exception: " + e.toString());
      e.printStackTrace();
      lib.shutdown();
    }
  }


  /**
   * Find the legal values for an ATLEAST predicate
   * @param propname The property's name
   * @param propvalue The property's ideal value
   * @param limit Maximal search depth (may be INF_DEPTH)
   * @return A map (value --> depth)
   */
  public AbstractMap findAtleastValues (String propname, String propvalue, int limit)
  {
    HashMap result = new HashMap();
    result.put (propvalue, new Integer (0));
    result.putAll (findOtherAtleastValues (propname, propvalue, limit, 1));
    return result;
  }


  /**
   * Recursive companion of findAtleastValues
   */
  public AbstractMap findOtherAtleastValues (String propname, String propvalue, int limit, int depth)
  {
    HashMap result = new HashMap();
    if (depth <= limit || limit == INF_DEPTH)
    {
      Vector rs = sqlQuerySimple (
        "SELECT DISTINCT propvalue FROM Taxonomy WHERE propname = '" +
        propname + "' AND parentvalue = '" + propvalue + "';", "propvalue");
      for (int i = 0; i < rs.size(); i++)
      {
        String v = (String) rs.get (i);
        result.put (v, new Integer (depth));
        result.putAll (findOtherAtleastValues (propname, v, limit, depth + 1));
      }
    }
    return result;
  }


  /**
   * Find the legal values for an ATMOST predicate
   * @param propname The property's name
   * @param propvalue The property's ideal value
   * @param limit Maximal search depth (may be INF_DEPTH)
   * @return A map (value --> depth)
   */
  public AbstractMap findAtmostValues (String propname, String propvalue, int limit)
  {
    HashMap result = new HashMap();
    result.put (propvalue, new Integer (0));
    result.putAll (findOtherAtmostValues (propname, propvalue, limit, 1));
    return result;
  }


  /**
   * Recursive companion of findAtmostValues
   */
  public AbstractMap findOtherAtmostValues (String propname, String propvalue, int limit, int depth)
  {
    HashMap result = new HashMap();
    if (depth <= limit || limit == INF_DEPTH)
    {
      Vector rs = sqlQuerySimple (
        "SELECT DISTINCT parentvalue FROM Taxonomy WHERE propname = '" +
        propname + "' AND propvalue = '" + propvalue + "' AND parentvalue IS NOT NULL;",
        "parentvalue");
      for (int i = 0; i < rs.size(); i++)
      {
        String v = (String) rs.get (i);
        result.put (v, new Integer (depth));
        result.putAll (findOtherAtmostValues (propname, v, limit, depth + 1));
      }
    }
    return result;
  }


  /**
   * Generate translated SQL for a single CRQL predicate
   * @param propname The property's name
   * @param type The kind of predicate (ATLEAST or ATMOST)
   * @param propvalue The property's ideal value
   * @param limit The maximal search depth (may be INF_DEPTH)
   * @return The generated SQL code
   */
  public String singleCRQLtoSQL (String propname, int type, String propvalue, int limit)
  {
    AbstractMap values = null;
    switch (type)
    {
      case ATLEAST:
        values = findAtleastValues (propname, propvalue, limit);
        break;
      case ATMOST:
        values = findAtmostValues (propname, propvalue, limit);
        break;
      default:
        throw new RuntimeException ("ERROR: unknown type in Library.singleCRQLtoSQL(String,int,String,int)!");
    }
    StringBuffer s = new StringBuffer();
    Iterator it = values.keySet().iterator();
    while (it.hasNext())
    {
      String value = (String) it.next();
      s.append ("'" + value + "', ");
    }
    s.delete (s.length() - 2, s.length()); // Remove last comma and space
    return "IN (" + s.toString() + ")";
  }


  /**
   * Find the table names and the names of the properties they're used for
   * @param query A query in CRQL
   * @return A map (table name --> property name)
   */
  public AbstractMap findTableNames (String query)
  {
    AbstractSet aliases = new HashSet();
    Matcher m1 = Pattern.compile ("Properties\\s+(\\w+)").matcher (query);
    while (m1.find()) aliases.add (m1.group (1));

    AbstractMap tableproperties = new HashMap();
    Matcher m2 = Pattern.compile ("(\\w+)\\.propname\\s*=\\s*\'(.+?)\'").matcher (query);
    while (m2.find())
    {
      String tablename = m2.group (1);
      String propname = m2.group (2);
      if (!aliases.contains (tablename))
        log.println ("Warning: table alias '" + tablename + "' is referenced to but not declared!");
      tableproperties.put (tablename, propname);
    }
    
    if (aliases.size() > tableproperties.size())
      log.println ("Warning: some table alias(es) are declared but not used in the query!");
    return tableproperties;
  }


  /**
   * Translate CRQL to ordinary SQL
   * @param query The input query
   * @param propnames A map (table name --> property name), as given by findTableNames
   * @return an array of two elements: first location holds the reritten query,
   * second location holds the list of CRQL predicates in the query (for use by other
   * methods)
   */
  public Object[] CRQLtoSQL (String query, AbstractMap propnames)
  {
    Matcher m = Pattern.compile ("(\\w+)\\.propvalue\\s+(ATLEAST|ATMOST)\\s+" +
    "\'(.+?)\'(\\s+WEIGHT\\s+([\\d\\.]+))?(\\s+LIMIT\\s+(\\d+))?").matcher (query);

    Vector predicates = new Vector();
    while (m.find())
    {
      String tablename = m.group (1);
      String type = m.group (2);
      String propvalue = m.group (3);
      int limit = INF_DEPTH;
      double weight = 1.0;
      if (m.group (4) != null) weight = Double.parseDouble (m.group (5));
      if (m.group (6) != null) limit = Integer.parseInt (m.group (7));
      CRQL predicate = new CRQL ();
      predicate.tablename = tablename;
      predicate.propname = (String) propnames.get (tablename);
      predicate.type = (type.equals ("ATLEAST") ? ATLEAST : ATMOST);
      predicate.propvalue = propvalue;
      predicate.weight = weight;
      predicate.limit = limit;
      predicates.add (predicate);
    }

    for (int i = 0; i < predicates.size(); i++)
    {
      m.reset (query);
      CRQL p = (CRQL) predicates.get (i);
      String newvalues = singleCRQLtoSQL (p.propname, p.type, p.propvalue, p.limit);
      query = m.replaceFirst (p.tablename + ".propvalue " + newvalues);
    }

    Object[] results = new Object [2];
    results[0] = query;
    results[1] = predicates;
    return results;
  }


  /**
   * Rank a single result
   * @param id The URI of the result
   * @param predicates A list of predicates (as returned by CRQLtoSQL)
   * @return A normalized rank in the range [0..1]
   */
  public double rank (String id, List predicates)
  {
    double totalrank = 0.0;
    double totalweight = 0.0;
    for (int i = 0; i < predicates.size(); i++)
    {
      CRQL crql = (CRQL) predicates.get (i);
      AbstractMap values = null;
      switch (crql.type)
      {
        case ATLEAST:
          values = findAtleastValues (crql.propname, crql.propvalue, crql.limit);
          break;
        case ATMOST:
          values = findAtmostValues (crql.propname, crql.propvalue, crql.limit);
          break;
        default:
          throw new RuntimeException ("ERROR: unknown type in Library.rank(String,List)!");
      }
      double r = rankOne (id, crql.propname, values);
      totalrank += (r * crql.weight);
      totalweight += crql.weight;
    }
    return totalrank / totalweight; // Weighted average
  }


  /**
   * Rank a single property of a single result
   * @param id The URI of the result
   * @param propname The property's name
   * @param values A map (property value --> depth)
   * @return A normalized rank in the range [0..1]
   */
  public double rankOne (String id, String propname, AbstractMap values)
  {
    Vector realvalues = sqlQuerySimple ("SELECT propvalue FROM Properties " +
      "WHERE uri = '" + id + "' AND propname = '" + propname + "';", "propvalue");
    int min_depth = INF_DEPTH;
    for (int i = 0; i < realvalues.size(); i++)
    {
      String value = (String) realvalues.get (i);
      int depth = ((Integer) values.get (value)).intValue();
      if (depth < min_depth || min_depth == INF_DEPTH) min_depth = depth;
    }
    if (min_depth == INF_DEPTH) return 0.0;
    else return 1.0 / (1.0 + min_depth); // Normalize to range [0..1]
  }


  /**
   * Execute a CRQL query, returning the complete result table
   * in the format of sqlQuery.
   */
  public Vector query (String expression)
  {
    // Part One: translate CRQL to SQL
    AbstractMap tablemap = findTableNames (expression);
    Object[] query_results = CRQLtoSQL (expression, tablemap);
    String sql_query = (String) query_results [0];
    List predicates = (List) query_results [1];
    Vector rows = sqlQuery (sql_query);

    // Part Two: reorder results of SQL query by rank (if applicable)
    if (predicates.size() > 0 && rows.size() > 0)
    {
      int uriIndex = ((Vector) rows.get (0)).indexOf ("URI");
      if (uriIndex == -1)
      {
        log.println ("Warning: no 'uri' column found in query result; cannot rank results!");
        return rows;
      }
      AbstractMap ranks = new HashMap();
      for (int i = 1; i < rows.size(); i++)
      {
        String thisuri = (String) (((Vector) rows.get (i)).get (uriIndex));
        double thisrank = rank (thisuri, predicates);
        ranks.put (thisuri, new Double (thisrank));
      }
      Vector header = (Vector) rows.remove (0);
      Collections.sort (rows, new RankComparator (ranks, uriIndex));
      rows.add (0, header);

      if (rows.size() == 1) log.println ("No results to rank.");
      else
      {
        log.println ("Ranking of the results:");
        java.text.DecimalFormat df = new java.text.DecimalFormat ("0.0000");
        for (int i = 1; i < rows.size(); i++)
        {
          String thisuri = (String) (((Vector) rows.get (i)).get (uriIndex));
          double thisrank = rank (thisuri, predicates);

          log.println ("#" + i + " is '" + thisuri + "' with rank " +
            df.format (thisrank));
        }
      }
      log.println();
    }
    return rows;
  }


  /**
   * Execute a query in SQL and return the results as a Vector of rows
   * <UL>
   * <LI>A row is represented as a Vector of String</LI>
   * <LI>The first entry contains the table names</LI>
   * <LI>The rest of the rows contain the table's contents</LI>
   * </UL>
   */
  public Vector sqlQuery (String expression)
  {
    Vector rows = new Vector();
    try
    {
      Statement stm = conn.createStatement();
      ResultSet rs = stm.executeQuery(expression);
      ResultSetMetaData meta = rs.getMetaData();
      int numcols = meta.getColumnCount();

      // Write table header
      Vector header = new Vector (numcols);
      for (int i = 0; i < numcols; i++)
        header.add((String) meta.getColumnName(i + 1));
      rows.add (header);

      // Write table contents
      while (rs.next())
      {
        Vector row = new Vector (numcols);
        for (int i = 0; i < numcols; i++)
          row.add (rs.getObject(i + 1));
        rows.add (row);
      }
      stm.close();
    }
    catch (SQLException ex) { }
    return rows;
  }



  /**
   * Singleton accessor; initialises library when first accessed
   * @return A reference to the library
   */
  public static Library instance()
  {
    if (instance == null) instance = new Library ();
    return instance;
  }


  /**
   * Singleton constructor
   */
  protected Library ()
  {
    try
    {
      Class.forName("org.hsqldb.jdbcDriver").newInstance();
      conn = DriverManager.getConnection("jdbc:hsqldb:" + databaseName, "sa", "");
      setupTables();
    }
    catch (Exception ex)
    {
      log.println ("Unexpected exception (while initializing database connection): " + ex.toString());
      ex.printStackTrace();
      shutdown(); // Prevents hanging of DB thread
    }
  }

  
  /**
   * Closes database connection
   */
  public void shutdown ()
  {
    if (conn != null)
    {
      sqlUpdateBlind ("SHUTDOWN COMPACT");
      try { conn.close(); } catch (Exception e) { }
      log.println ("Database closed.");
      log.close();
      conn = null;
    }
  }


  public void finalize() throws Throwable { shutdown(); }


  /**
   * Use a different log stream
   * @param stream The stream receiving the logging information, eg. System.out
   */
  public void setLog (PrintStream stream) { log = stream; }



  /**
   * Delete a stored query (if found)
   * @param name The query's name
   */
  public void deleteQuery (String name)
  {
    sqlUpdateBlind ("DELETE FROM SavedQueries WHERE name = '" + name + "'");
  }


  /**
   * Save a given query in the database
   * @param name The name you want to give to the query (avoid spaces and the like!)
   * @param query The query itself
   */
  public void saveQuery (String name, String query)
  {
    query = query.replaceAll ("\'", "\'\'"); // Escape quotation marks to avoid interpretation by SQL
    query = query.replaceAll ("\\\\", "\\\\\\\\"); // Encode backslashes
    query = query.replaceAll ("\\n", " \\\\n"); // Encode newlines. Extra space is required for decoding!
    deleteQuery (name);
    sqlUpdateBlind ("INSERT INTO SavedQueries (name, query) VALUES ('" + name + "', '" + query + "');");

    // Note: uncommenting the following lines enables creating a 'view' for the saved query. 
    /*sqlUpdateBlind ("DROP VIEW " + name + " IF EXISTS;");
    sqlUpdateBlind ("CREATE VIEW " + name + " AS " + query);*/
  }

  
  /**
   * Retrieve a saved query with a given name.
   * Returns an empty string if the query is not found.
   * @param name The query's name
   * @return The stored query if it's found, the empty string otherwise
   */
  public String loadQuery (String name)
  {
    Vector queryResults = sqlQuerySimple ("SELECT query FROM SavedQueries WHERE name = '" + name + "'", "query");
    if (queryResults.size() < 1) return "";
    else
    {
      String query = (String) queryResults.get(0);
      query = query.replaceAll (" \\\\n", "\n");      // Decode newlines
      query = query.replaceAll ("\\\\\\\\", "\\\\");  // Decode backslashes
      return query; // Note: decoding quotes is not necessary, as it is a query *result*.
    }
  }


  /**
   * Execute a SQL command that doesn't return any useful results, like INSERT or DELETE
   * @param expression A SQL expression
   * @throws SQLException if an error occurs while updating
   */
  public synchronized void sqlUpdate (String expression) throws SQLException
  {
    Statement stm = conn.createStatement();
    int i = stm.executeUpdate(expression);
    if (i == -1) log.println("Database error (on update): " + expression);
    stm.close();
  }

  
  /**
   * Identical in behaviour to sqlUpdate, but ignores any exceptions.
   * Especially useful when lots of updates have to happen, and you
   * don't want an error in one of them to hamper the control flow.
   * @param expression A SQL expression
   * @see #sqlUpdate(String)
   */
  public void sqlUpdateBlind (String expression)
  {
    try { sqlUpdate (expression); }
    catch (SQLException e)
    {
      log.println ("Ignored exception (on blind update): " + e.toString());
    }
  }


  /**
   * Execute a SQL query, returning a single column
   * @param expression The query
   * @colname The name of the column that should form the result
   * @return A Vector of String, containing the values of the given column
   */
  public Vector sqlQuerySimple (String expression, String colname)
  {
    Vector result = new Vector();
    try
    {
      Statement stm = conn.createStatement();
      ResultSet rs = stm.executeQuery(expression);
      int colnum = rs.findColumn (colname);
      while (rs.next()) result.add (rs.getObject (colnum));
      stm.close();
    }
    catch (SQLException ex) { } // Return empty vector if column doesn't exist
    return result;
  }


  
  /**
   * (Re)Indexes all components
   */
  public void indexComponents() { refreshComponents (true); }


  /**
   * @param updateAll If true, all components are indexed, otherwise only
   * the modified components are updated
   */
  public void refreshComponents (boolean updateAll)
  {
    sqlUpdateBlind("DELETE FROM Library");
    sqlUpdateBlind("DELETE FROM Properties");

    // Get a list of all elegible component files
    // Note that we can append results by passing the same 'files' parameter
    LinkedList files = new LinkedList();
    getFileTree (new File("./models"),          ".xml", files); 
    getFileTree (new File("./repository"),      ".xml", files); 
    getFileTree (new File("./implementations"), ".xml", files); 
    Indexer idx = new Indexer (files, updateAll);
    idx.start();
  }


  /**
   * Get a file tree (files in given directory and all subdirs)
   * @param dir The topmost directory
   * @param ext The suffix of the valid files (e.g. ".xml")
   * @param result The resulting list of filenames
   */
  public static void getFileTree(File dir, String ext, List result)
  {
    if (!dir.exists()) return;

    // Get all .xml files in this directory
    DirFileFilter filter = new DirFileFilter(ext);
    filter.setAcceptDirs(false);
    filter.setAcceptFiles(true);
    File files[] = dir.listFiles(filter);

    // Get all directories in this directory
    filter.setAcceptDirs(true);
    filter.setAcceptFiles(false);
    File dirs[] = dir.listFiles(filter);

    // Add files from this level
    for (int i = 0; i < files.length; i++)
      result.add (files[i]);

    // Recurse into the subdirectories
    for (int i = 0; i < dirs.length; i++)
      getFileTree(dirs[i], ext, result);
  }


  /**
   * Indexes a single component
   * @param file The XML file containing the component
   * @return true if the operation completes successfully, false otherwise
   */
  public void indexComponent (File file)
  {
    String name = file.getName();
    if (name.endsWith(".xml")) name = name.substring(0, name.length() - 4);
    String path = file.getPath();

    // Determine the kind of component and extract its CoCompose-specific properties
    try
    {    
      Model model = new Model(path, name); // Load the model
      if (model.getFreeNodeCount() == 1)   // It's a Concept or a Composite
      {
        Concept concept = model.firstConcept();
        if (concept != null)
          indexConcept (path, concept);
        else
        {
          Composite composite = model.firstComposite();
          if (composite != null) indexComposite (path, composite);
          // Else it's some solitary element of unknown type
        }
      }
      else indexSolutionPattern (path, model); // >1 element, so assume solution pattern.
    }
    catch (Exception exc)
    {
      //log.println("Error while parsing XML file: " + exc.getMessage());
      // Assume unknown files are implementation generators
      sqlUpdateBlind("INSERT INTO Library VALUES ('"
        + path + "', '" + name + "', 'implementation-generator')");
    }

    // Now process the eventual file containing the CoCompose-independent facets
    String facetfilename = path;
    if (facetfilename.endsWith(".xml"))
      facetfilename = facetfilename.substring(0, facetfilename.length() - 4);
    facetfilename = facetfilename + ".properties";
    Properties props = new Properties ();
    try
    {
      props.load (new FileInputStream (facetfilename));
      addFacets (path, props);
    }
    catch (IOException ioexc) {} // No problem if facet file doesn't exist, it's optional
  }

 
  /**
   * Subfunction to index the given Composite
   * @param uri The Composite's filename
   * @param c The Composite to index
   */
  protected void indexComposite (String uri, Composite c)
  {
    String name = c.getType();
    if (name == "") return;
        
    sqlUpdateBlind("INSERT INTO Library VALUES ('"
      + uri + "', '" + name + "', 'composite-type')");
    Enumeration pubconcs = c.getPublishedConcepts();
    while (pubconcs.hasMoreElements())
    {
      PublishedConcept pubconc = (PublishedConcept) pubconcs.nextElement();
      if (pubconc.getPublicName() != "")
        sqlUpdateBlind("INSERT INTO Properties VALUES ('"
          + uri + "', 'published-concept', '" + pubconc.getPublicName() + "')");
    }
    Enumeration roles = c.getRoles();
    while (roles.hasMoreElements())
    {
      CompositeRole role = (CompositeRole) roles.nextElement();
      sqlUpdateBlind("INSERT INTO Properties VALUES ('"
        + uri + "', 'role', '" + role.getPublicName() + "')");
    }
    Enumeration solutions = c.getSolutions();
    while (solutions.hasMoreElements())
    {
      SolutionPattern solution = (SolutionPattern) solutions.nextElement();
      sqlUpdateBlind("INSERT INTO Properties VALUES ('"
        + uri + "', 'solution-pattern', '" + solution.getFilename() + "')");
    }
    Enumeration implementations = c.getImplementations();
    while (implementations.hasMoreElements())
    {
      ImplementationGenerator implementation = (ImplementationGenerator) implementations.nextElement();
      sqlUpdateBlind("INSERT INTO Properties VALUES ('"
        + uri + "', 'implementation-generator', '" + implementation.getFilename() + "')");
    }
  }


  /**
   * Subfunction to index the given Concept
   * @param uri The concept's filename
   * @param c The concept to index
   */
  protected void indexConcept (String uri, Concept c)
  {
    String name = c.getPublicName();
    if (name == "") return;
    sqlUpdateBlind("INSERT INTO Library VALUES ('"
      + uri + "', '" + name + "', 'concept')");
    Enumeration impls = c.getImplementations();
    while (impls.hasMoreElements())
    {
      Implementation impl = (Implementation) impls.nextElement();
      sqlUpdateBlind("INSERT INTO Properties VALUES ('"
        + uri + "', 'implementation', '" + impl.getLanguage() + "."
        + impl.getConceptForm() + "')");
    }
  }


  /**
   * Subfunction to index the given SolutionPattern (or plain vanilla Model)
   * @param uri The solution's filename
   * @param m The solution to index
   */
  protected void indexSolutionPattern (String uri, Model m)
  {
    String name = m.getName();
    if (name == "") return;
    sqlUpdateBlind("INSERT INTO Library VALUES ('"
      + uri + "', '" + name + "', 'solution-pattern')");
    Iterator abstrconcs = m.getAbstractConcepts().iterator();
    while (abstrconcs.hasNext())
    {
      Object o = abstrconcs.next();
      if (o instanceof SolutionRole)
      {
        SolutionRole role = (SolutionRole) o;
        sqlUpdateBlind("INSERT INTO Properties VALUES ('"
          + uri + "', 'role', '" + role.getPublicName() + "')");
      }
    }
  }


  /**
   * Subfunction to add the facets from the given Properties object
   * to the database
   * @param uri The filename
   * @param props The Properties object containing the facets
   */
  protected void addFacets (String uri, Properties props)
  {
    Enumeration propnames = props.propertyNames();
    while (propnames.hasMoreElements())
    {
      String key = (String) propnames.nextElement();
      String values = props.getProperty (key);
      try
      {
        String[] separateValues = values.split("\\s");
        for (int i = 0; i < separateValues.length; i++)
          sqlUpdateBlind ("INSERT INTO Properties VALUES ('"
            + uri + "', '" + key + "', '" + separateValues[i] + "')");
      }
      catch (PatternSyntaxException pse)
      {
        log.println ("Library error (while processing file '" + uri + "': " + pse.toString());
      }
    }
  }


  /**
   * Create the DB tables (if necessary, for example when first used), and
   * connect them to their physical (text) files.
   */
  protected void setupTables ()
  {
    if (conn == null) return;
    try
    {
      if (tableSize ("Library") == -1)
        sqlUpdate ("CREATE TEXT TABLE Library " +
                   "( uri VARCHAR(255), name VARCHAR(255), type VARCHAR(255), " +
                   "UNIQUE (uri) )");
      sqlUpdate ("SET TABLE Library SOURCE \"" + tableLibraryFilename + ";fs=\\semi\"");

      if (tableSize ("Properties") == -1)
        sqlUpdate ("CREATE TEXT TABLE Properties " +
                   "( uri VARCHAR(255), propname VARCHAR(255), propvalue VARCHAR(255), " +
                   "UNIQUE (uri, propname, propvalue) )");
      sqlUpdate ("SET TABLE Properties SOURCE \"" + tablePropertiesFilename + ";fs=\\semi\"");

      if (tableSize ("Taxonomy") == -1)
        sqlUpdate ("CREATE TEXT TABLE Taxonomy " + 
                   "( propname VARCHAR(255), propvalue VARCHAR(255), parentvalue VARCHAR(255), " +
                   "UNIQUE (propname, propvalue, parentvalue) )");
      sqlUpdate ("SET TABLE Taxonomy SOURCE \"" + tableTaxonomyFilename + ";fs=\\semi\"");

      if (tableSize ("SavedQueries") == -1)
        sqlUpdate ("CREATE TEXT TABLE SavedQueries " +
                   "( name VARCHAR(255), query VARCHAR(255), " +
                   "UNIQUE (name) )");
      sqlUpdate ("SET TABLE SavedQueries SOURCE \"" + tableSavedQueriesFilename + ";fs=\\semi\"");

    }
    catch (SQLException ex)
    {
      log.println ("Database error (while creating tables): " + ex.toString());
    }
  }


  /**
   * Check how many rows the given table contains, and if it exists
   * @param name The table name to check
   * @return If the table with the given name exists return its number of rows,
   * else return -1
   */
  public int tableSize (String name)
  {
    try
    {
      Statement stm = conn.createStatement();
      ResultSet rs = stm.executeQuery("SELECT COUNT(*) FROM " + name);
      int val = -2;
      if (rs.next()) val = rs.getInt (1);
      stm.close();
      return val;
    }
    catch (SQLException se) { return -1; }
  }
} // class Library